#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>

#define EV_SYN	0x00
#define EV_KEY	0x01
#define EV_REL	0x02
#define EV_ABS	0x03
#define EV_MSC	0x04

#define SYN_REPORT	0
#define BTN_LEFT	0x110
#define BTN_EXTRA	0x114
#define ABS_X		0x00
#define ABS_Y		0x01
#define ABS_RX		0x03
#define ABS_RY		0x04

typedef struct tagevent
{
    struct timeval time;
    unsigned short type;
    unsigned short code;
    int value;
} input_event;

void ParseEvent( input_event ev )
{
    static unsigned short X, Y, State;
    static unsigned short X2, Y2, State2;
    static unsigned char bIsSecond;

    if( ev.type == EV_KEY ) {
        if( ev.code == BTN_LEFT ) { // Mouse/FirstPoint state
            State = ev.value;
            bIsSecond = 0;
        } else if( ev.code == BTN_EXTRA ) { // Second point state
            State2 = ev.value;
            bIsSecond = 1;
        }
    } else if( ev.type == EV_ABS ) {
        if( ev.code == ABS_X ) { // The X position of first point
            X = ev.value;
            bIsSecond = 0;
        } else if( ev.code == ABS_Y ) { // The Y position of first point
            Y = ev.value;
            bIsSecond = 0;
        } else if( ev.code == ABS_RX ) { // The X position of second point
            X2 = ev.value;
            bIsSecond = 1;
        } else if( ev.code == ABS_RY ) { // The Y postiion of second point
            Y2 = ev.value;
            bIsSecond = 1;
        }
    } else if( ev.type == EV_SYN && ev.value == SYN_REPORT ) { // A Synchronization signal, it means all data will be valid.
        if( bIsSecond ) { // Second point
            printf("Point%d: State %X, X %d, Y %d\n", bIsSecond, State2 & 0x00FF, X2, Y2);
        } else { // First point
            printf("Point%d: State %X, X %d, Y %d\n", bIsSecond, State & 0x00FF, X, Y);
        }
    }
}

int main( int argc, char **argv )
{
    int fd;
    unsigned char byFlag = 0; /* 0: Point */
    char strPort[64] = { 0 };
    fd_set readfds;

    if( argc == 1 || argc > 2 ) {
        printf(" \nUsage:\n \tevent [Port]\n");
        printf("Example:\n \tevent /dev/input/event1\n\n");
        return 0;
    }
	
    sprintf( strPort, "%s", argv[1] );
    printf("Event Port = %s ", strPort);
    fd = open(strPort, O_RDONLY );
    if( fd >= 0 ) {	
        printf(" open: ok\n" );
        FD_ZERO( &readfds );
        FD_SET( fd , &readfds );
        while( 1 ) {
            if( select( fd+1, &readfds, NULL, NULL, NULL ) > 0 ) {
                input_event ev;
                if( sizeof( input_event ) == read( fd, &ev, sizeof( input_event ) ) ) {
                    ParseEvent( ev );
                } else {
                    printf(" Nothing read \n" );
                }
            }
        }	
    }
    return 0;
}
